from .create_browser import *

import appdirs
import logging
# import pickle
import dill as pickle
import selenium.common.exceptions
import selenium.webdriver
import sys
import time
import traceback
import os

USER_AGENT = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Trident/5.0)"
REFUSE_AFTER_EXCEPTION = True
EXECUTOR_PORT = 4444
REMOTE_EXECUTOR = 'http://127.0.0.1:%s/wd/hub'

# PICKLE_FILENAME = '/tmp/nhsnwebdriverdump'
# PICKLE_FILENAME = os.path.join(
#     appdirs.user_data_dir('wabot'),
#     'saved_browser_instances.pickle'
# )

LOGGER = logging.getLogger('wabot')

DEFAULT_WEBDRIVER_TYPE = 'firefox1'

class BrowserProxy:
    def __init__(
        self,
        session_name='webdriver',
        pickle_filename=None,
        phantom=False,
        webdriver_type=None # remote_chromium2
    ):
        """
        BrowserProxy wraps a selenium webdriver instance and provides utility
        functions for automation webpages.
        """
        LOGGER.info('requesting selenium browser instance (%s): instance_name = %s', webdriver_type, session_name)

        # if pickle_filename is None:
        #     pickle_filename = PICKLE_FILENAME

        # self._pickle_filename = pickle_filename

        if webdriver_type is None:
            webdriver_type = 'firefox1'

        assert webdriver_type in (
            'firefox1',
            'firefox2',
            'chromium2',
            'remote_chromium2',
            'phantomjs'
        ), 'webdriver_type must be firefox1, firefox2, chromium2, remote_chromium2, or phantomjs'




        try:
            self.driver_type = webdriver_type
            if phantom:
                pass
                # driver_type = "phantomjs"
            self.driver = self.get_driver(webdriver_type, session_name)
            if not self.driver:
                LOGGER.error('failed to get selenium webdriver')
                self.good = False
                return
        except Exception as ex:
            print('caught exception at BrowserProxy().__init__')
            print(type(ex), ex)
            raise

        # self.page = nhsn_lo.pages.Login(self)
        # self.good = True

    def get_page(self):
        return type(self.page)

    def set_page(self, page):
        LOGGER.trace('switching page to %s' % page)
        newpage = page(self)
        if hasattr(newpage, 'verify'):
            rc = newpage.verify()
            if not rc:
                LOGGER.error('failed to verify page: %s' % page)
                # input('failure')
                return False
            else:
                LOGGER.trace('verified page: %s' % page)

        self.page = newpage
        return True

    def switch_after(self, fn=None, page=None):
        if not fn:
            return lambda fn: self.switch_after(fn, page = page)

        def wrapper(*args, **kwargs):
            self.set_page(page)
            return fn(*args, **kwargs)
            
        return wrapper

    def __getattr__(self, name):
        """
        Access method on current page.

        When object.method is referenced, python invokes
        object.__getattribute__ which checks the instance
        for a matching property, If none is found, python
        invokes object.__getattr__, which is overridden here.
        If the attribute is not found in the main class, this
        will search the page object for the method. This
        allows invoking page methods through the proxy class.

        nhsn.foo() invokes nhsn.page.foo()
        """
        # todo(mg) would this be a good spot to inject page verification?
        # idea: maybe we could decorate method with error checking and
        # error collection? method can return value, rc and this can
        # queue actions and error check them as it goes? At the end,
        # the user would be able to pull out the errors and check them
        if REFUSE_AFTER_EXCEPTION and not self.good:
            LOGGER.warn("broken state - refusing to invoke page action: %s" % (name))
            return

        try:
            method = getattr(self.page, name)
            return method
        except AttributeError as ex:
            raise

    def __getitem__(self, key):
        """
        Access element on the current page.
        """
        return self.page[key]

    def perform(self, meth, *args, **kwargs):
        """
        Calls self.page.method and catches exceptions.
        """
        # input('<breakpoint> at perform')
        if REFUSE_AFTER_EXCEPTION and not self.good:
            LOGGER.warn("broken state - refusing to invoke page action: %s" % (meth))
            return

        try:
            x = getattr(self.page, meth)
        except AttributeError as ex:
            LOGGER.error("Failed to invoke page action. "\
                                 "page = %s, method = %s" %\
                                 (self.page.__class__, meth))
            return None

        # Hooks:
        # x = self.page.detect_logged_out()
        # y = self.page.detected_logged_in()
        # print('logged out/in: ', x, y)

        LOGGER.trace("invoking %s.%s" % (self.page.__class__.__name__, meth))
        try:
            return x(*args, **kwargs)
        except selenium.common.exceptions.NoSuchWindowException:
            self.good = False
        except Exception as ex:
            print("caught exception: %s" % (type(ex)))
            exc_type, exc_value, exc_traceback = sys.exc_info()
            lines = traceback.format_exception(exc_type, exc_value,
                exc_traceback)
            for line in lines:
                print(line, end = "")

            self.good = False

    def reset_good_status(self):
        self.good = True

    def get_driver(self, browser, session_name='webdriver'):
        """
        Returns selenium objects of different types

        The combinations of selenium{1,2} {chrome,firefox} are supported.
        In the case of selenium 1 (selenium server), the host is a constant
        defined at the top of this file.

        Args:
            browser: The selenium webdriver requested.
                     If a chromium2 webdriver is requested, selenium
                     will try to use chromium as the browser. If
                     firefox2 is requested, selenium will try to use
                     firefox. Chromium is very fast but it is designed
                     to be fast for an interactive user which means it
                     is fast at the cost of processing power and
                     ram. Firefox is about as fast but uses less
                     resources.

                     Available browser drivers are:
                     chromium2, chromium1, firefox2, firefox1

                     The associated webdriver has to be installed
                     and runnable on the system.
        Returns:
            The selenium webdriver handle.
        """
        LOGGER.debug('requesting selenium browser instance: type = %s' % (browser))

        driver = None
        browser_factory = CreateBrowser()
        if browser == 'chromium2': # Selenium 2 - Chrome
            driver = self._create_driver_chromium2()

        elif browser == 'remote_chromium2':
            driver = browser_factory._create_driver_remote_chromium2(session_name)
            # driver = self._create_driver_remote_chromium2(session_name)

        elif browser == 'chromium1': # Selenium 1 - Chrome without working user agent switch
            driver = self._create_driver_chromium1()

        elif browser == 'firefox2': # Selenium 2 - Firefox
            driver = self._create_driver_firefox2()

        elif browser == 'firefox1': # Selenium 1 - Firefox
            driver = self._create_driver_firefox1()

        elif browser == 'phantomjs':
            driver = self._create_driver_phantomjs()
        else:
            LOGGER.error(
                'an attempt was made to request an '\
                'unsupported (by this product) selenium '\
                'webdriver; refusing. requested = %s'\
                % (browser)
            )
            
        driver.implicitly_wait(10)
        return driver

